"""This module contains boilerplate json helpers."""
import datetime as dt
import json
from decimal import Decimal

from ._datetime import date_to_iso8601
from ._datetime import datetime_to_iso8601

__all__ = ["JSONEncoder", "read_json", "iter_json"]


class JSONEncoder(json.JSONEncoder):
    """Extend JSONEncoder to gracefully handle various primitives."""

    def default(self, o):
        """Convert data into a serializable format."""
        if isinstance(o, dt.datetime):
            return datetime_to_iso8601(o)
        elif isinstance(o, dt.date):
            return date_to_iso8601(o)
        elif isinstance(o, dt.time):
            representation = o.isoformat()
            if o.microsecond:
                return representation[:12]
            return representation
        elif isinstance(o, Decimal):
            return float(o)
        elif hasattr(o, "__getitem__"):
            return dict(o)
        elif hasattr(o, "__iter__"):
            return tuple(item for item in o)
        return super(JSONEncoder, self).default(o)


def read_json(filename):
    """Read a JSON file.

    **Example**:

        >>> read_json('/path/to/data.json')
        [{ 'name': 'foo' }]

    :param filename:
        Path to JSON file.
    :return:
        A Python representation of the JSON document.
    """
    with open(filename) as fh:
        return json.loads(fh.read())


def iter_json(filename):
    """Iterate a JSON file containing a list of dictionaries.

    **Example**:

        >>> for item in iter_json('/path/to/data.json'):
        ...    print(item)
        [{ 'name': 'foo' }]

    :param filename:
        Path to JSON file.
    :yield:
        A Python ``dict`` representation of a JSON object.
    """
    with open(filename) as fh:
        for item in json.loads(fh.read()):
            yield item



"""This module contains boilerplate urllib helpers."""
import urllib

__all__ = ["update_qs"]


def update_qs(url, **kwargs):
    """Update a query string."""
    parts = list(urllib.parse.urlparse(url))
    queryargs = urllib.parse.parse_qs(parts[4], keep_blank_values=False)
    for k, v in kwargs.items():
        if v is None:
            del queryargs[k]
        else:
            queryargs[k] = v
    parts[4] = urllib.parse.urlencode(queryargs, doseq=True)
    return urllib.parse.urlunparse(parts)



"""This module contains boilerplate random helpers."""
import random
from string import ascii_letters
from string import digits

__all__ = ["random_string"]


def random_string(n):
    """Generate a random alphanumeric string of size ``n``."""
    symbols = ascii_letters + digits
    return "".join(random.choice(symbols) for i in range(n))



"""This module contains boilerplate datetime helpers."""
import calendar
import datetime as dt

__all__ = [
    "iso8601_to_datetime",
    "date_to_iso8601",
    "datetime_to_iso8601",
    "datetime_to_unixtimestamp",
]


def iso8601_to_datetime(strtime):
    """Convert ISO 8601 strings to datetime instance."""
    try:
        return dt.datetime.strptime(strtime, "%Y-%m-%dT%H:%M:%S.%fZ")
    except ValueError:
        return dt.datetime.strptime(strtime, "%Y-%m-%dT%H:%M:%SZ")


def datetime_to_iso8601(datetime):
    """Convert datetime to ISO 8601 string."""
    if datetime.microsecond > 0:
        return datetime.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return datetime.strftime("%Y-%m-%dT%H:%M:%SZ")


def date_to_iso8601(date):
    """Convert date to ISO 8601 string."""
    datetime = dt.datetime.combine(date, dt.datetime.min.time())
    if datetime.microsecond > 0:
        return datetime.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return datetime.strftime("%Y-%m-%dT%H:%M:%SZ")


def datetime_to_unixtimestamp(datetime):
    """Convert datetime to ISO 8601 string."""
    return int(calendar.timegm(datetime.timetuple()))



# flake8: noqa
__title__ = "yakutils"
__version__ = "1.1.9"
__author__ = "Nick Ficano"
__license__ = "MIT License"
__copyright__ = "Copyright 2020 Nick Ficano"

from ._csv import iter_csv
from ._csv import read_csv
from ._datetime import date_to_iso8601
from ._datetime import datetime_to_iso8601
from ._datetime import datetime_to_unixtimestamp
from ._datetime import iso8601_to_datetime
from ._hashlib import md5
from ._hashlib import sha1
from ._json import iter_json
from ._json import JSONEncoder
from ._json import read_json
from ._logging import SANE_BASE_LOGGING_CONFIG
from ._logging import setup_logging
from ._random import random_string
from ._urllib import update_qs
